@@ -142,8 +142,6 @@ static int do_binauth(
 	char password_enc[64] = {0};
 	char custom_dec_b64[384 *4 / 3] = {0};
 	char custom_enc[384] = {0};
-	char *lockfile;
-	FILE *fd;
 	char redirect_url_enc_buf[QUERYMAXLEN] = {0};
 	const char *username;
 	const char *password;
@@ -157,7 +155,7 @@ static int do_binauth(
 	unsigned long long int download_rate;
 	unsigned long long int upload_quota;
 	unsigned long long int download_quota;
-	int rc;
+	int rc =1;
 	s_config *config;
 
 	config = config_get_config();
@@ -185,7 +183,7 @@ static int do_binauth(
 	} else {
 
 		if (!custom || strlen(custom) == 0) {
-			custom="bmE";
+			custom="bmE=";
 		}
 		uh_b64decode(custom_dec_b64, sizeof(custom_dec_b64), custom, strlen(custom));
 		uh_urlencode(custom_enc, sizeof(custom_enc), custom_dec_b64, strlen(custom_dec_b64));
@@ -214,28 +212,16 @@ static int do_binauth(
 	debug(LOG_DEBUG, "BinAuth argv: %s", argv);
 
 	// ndsctl will deadlock if run within the BinAuth script so lock it
+	if(ndsctl_lock() == 0) {
+		// execute the script
+		rc = execute_ret_url_encoded(msg, sizeof(msg) - 1, argv);
+		debug(LOG_DEBUG, "BinAuth returned arguments: %s", msg);
+		free(argv);
 
-	safe_asprintf(&lockfile, "%s/ndsctl.lock", config->tmpfsmountpoint);
-
-	if ((fd = fopen(lockfile, "r")) == NULL) {
-		//No lockfile, so create one
-		fd = fopen(lockfile, "w");
+		// unlock ndsctl
+		ndsctl_unlock();
 	}
 
-
-	// execute the script
-	rc = execute_ret_url_encoded(msg, sizeof(msg) - 1, argv);
-	debug(LOG_DEBUG, "BinAuth returned arguments: %s", msg);
-	free(argv);
-
-	// unlock ndsctl
-	if (fd) {
-		fclose(fd);
-		remove(lockfile);
-	}
-
-	free(lockfile);
-
 	if (rc != 0) {
 		debug(LOG_DEBUG, "BinAuth script failed to execute");
 		return 0;
@@ -515,7 +501,6 @@ static int try_to_authenticate(struct MHD_Connection *connection, t_client *clie
 {
 	s_config *config;
 	const char *tok;
-	char hid[128] = {0};
 	char rhid[128] = {0};
 	char *rhidraw = NULL;
 
@@ -533,8 +518,7 @@ static int try_to_authenticate(struct MHD_Connection *connection, t_client *clie
 		//Check if token (tok) or hash_id (hid) mode
 		if (strlen(tok) > 8) {
 			// hid mode
-			hash_str(hid, sizeof(hid), client->token);
-			safe_asprintf(&rhidraw, "%s%s", hid, config->fas_key);
+			safe_asprintf(&rhidraw, "%s%s", client->hid, config->fas_key);
 			hash_str(rhid, sizeof(rhid), rhidraw);
 			free (rhidraw);
 			if (tok && !strcmp(rhid, tok)) {
@@ -636,7 +620,7 @@ static int authenticate_client(struct MHD_Connection *connection,
 		seconds = (60 * config->session_timeout);
 	}
 
-	debug(LOG_DEBUG, "timout seconds: %d", seconds);
+	debug(LOG_DEBUG, "timeout seconds: %d", seconds);
 
 	if (seconds != (60 * config->session_timeout)) {
 		client->session_end = (client->session_start + seconds);
@@ -691,12 +675,12 @@ static int authenticated(struct MHD_Connection *connection,
 						t_client *client)
 {
 	s_config *config = config_get_config();
-	const char *host = NULL;
+	const char *host = config->gw_address;
 	char redirect_to_us[128];
 	char *fasurl = NULL;
 	char query_str[QUERYMAXLEN] = {0};
 	char *query = query_str;
-	char msg[HTMLMAXSIZE] = {0};
+	char *msg;
 	char clientif[64] = {0};
 	int rc;
 	int ret;
@@ -711,6 +695,11 @@ static int authenticated(struct MHD_Connection *connection,
 		debug(LOG_DEBUG, "An authenticated client is requesting: host [%s] url [%s]", host, url);
 	}
 
+	if (host == NULL) {
+		debug(LOG_ERR, "authenticated: Error getting host");
+		host = config->gw_address;
+	}
+
 	/* check if this is a late request, meaning the user tries to get the internet, but ended up here,
 	 * because the iptables rule came too late
 	 */
@@ -777,15 +766,16 @@ static int authenticated(struct MHD_Connection *connection,
 
 	// User just entered gatewayaddress:gatewayport so give them the info page
 	if (strcmp(url, "/") == 0) {
-		rc = execute_ret(msg, HTMLMAXSIZE - 1, "/usr/lib/opennds/client_params.sh '%s'", client->ip);
+		msg = safe_malloc(HTMLMAXSIZE);
+		rc = execute_ret(msg, HTMLMAXSIZE - 1, "%s '%s'", config->status_path, client->ip);
 
 		if (rc != 0) {
-			debug(LOG_WARNING, "Script: /usr/lib/opennds/client_params.sh - failed to execute");
+			debug(LOG_WARNING, "Script: %s - failed to execute", config->status_path);
 			return 0;
 		}
 
 		// serve the script output (in msg)
-		response = MHD_create_response_from_buffer(strlen(msg), (char *)msg, MHD_RESPMEM_MUST_COPY);
+		response = MHD_create_response_from_buffer(strlen(msg), (char *)msg, MHD_RESPMEM_MUST_FREE);
 
 		if (!response) {
 			return send_error(connection, 503);
@@ -807,7 +797,8 @@ static int authenticated(struct MHD_Connection *connection,
 static int show_preauthpage(struct MHD_Connection *connection, const char *query)
 {
 	s_config *config = config_get_config();
-	char msg[HTMLMAXSIZE] = {0};
+
+	char *msg;
 	const char *user_agent = NULL;
 	char enc_user_agent[256] = {0};
 	char *preauthpath = NULL;
@@ -824,6 +815,7 @@ static int show_preauthpage(struct MHD_Connection *connection, const char *query
 
 	if (strcmp(preauthpath, config->fas_path) == 0) {
 		free (preauthpath);
+
 		MHD_get_connection_values(connection, MHD_HEADER_KIND, get_user_agent_callback, &user_agent);
 		uh_urlencode(enc_user_agent, sizeof(enc_user_agent), user_agent, strlen(user_agent));
 		debug(LOG_DEBUG, "PreAuth: Encoded User Agent is [ %s ]", enc_user_agent);
@@ -831,17 +823,20 @@ static int show_preauthpage(struct MHD_Connection *connection, const char *query
 		uh_urlencode(enc_query, sizeof(enc_query), query, strlen(query));
 		debug(LOG_DEBUG, "PreAuth: Encoded query: %s", enc_query);
 
+		msg = safe_malloc(HTMLMAXSIZE);
+
 		safe_asprintf(&cmd, "%s '%s' '%s' '%d' '%s'", config->preauth, enc_query, enc_user_agent, config->login_option_enabled, config->themespec_path);
 		rc = execute_ret_url_encoded(msg, HTMLMAXSIZE - 1, cmd);
 		free(cmd);
 
 		if (rc != 0) {
 			debug(LOG_WARNING, "Preauth script - failed to execute: %s, Query[%s]", config->preauth, query);
+			free(msg);
 			return send_error(connection, 511);
 		}
 
 		// serve the script output (in msg)
-		response = MHD_create_response_from_buffer(strlen(msg), (char *)msg, MHD_RESPMEM_MUST_COPY);
+		response = MHD_create_response_from_buffer(strlen(msg), (char *)msg, MHD_RESPMEM_MUST_FREE);
 
 		if (!response) {
 			return send_error(connection, 503);
@@ -868,7 +863,8 @@ static int preauthenticated(struct MHD_Connection *connection,
 							const char *url,
 							t_client *client)
 {
-	const char *host = NULL;
+	s_config *config = config_get_config();
+	const char *host = config->gw_address;
 	const char *redirect_url;
 	char query_str[QUERYMAXLEN] = {0};
 	char *query = query_str;
@@ -877,7 +873,6 @@ static int preauthenticated(struct MHD_Connection *connection,
 	char originurl[QUERYMAXLEN] = {0};
 
 	int ret;
-	s_config *config = config_get_config();
 
 	ret = MHD_get_connection_values(connection, MHD_HEADER_KIND, get_host_value_callback, &host);
 
@@ -888,6 +883,11 @@ static int preauthenticated(struct MHD_Connection *connection,
 
 	debug(LOG_DEBUG, "preauthenticated: host [%s] url [%s]", host, url);
 
+	if (host == NULL) {
+		debug(LOG_ERR, "preauthenticated: Error getting host");
+		host = config->gw_address;
+	}
+
 	// User just accessed gatewayaddress:gatewayport either directly or by redirect
 	if (strcmp(url, "/") == 0) {
 		if (strcmp(host, config->gw_address) == 0 || strcmp(host, config->gw_ip) == 0 || strcmp(host, config->gw_fqdn) == 0) {
@@ -1071,10 +1071,14 @@ static int redirect_to_splashpage(struct MHD_Connection *connection, t_client *c
  */
 static char *construct_querystring(t_client *client, char *originurl, char *querystr ) {
 
-	char hash[128] = {0};
+	char cid[87] = {0};
+	char msg[16] = {0};
 	char clientif[64] = {0};
 	char query_str[QUERYMAXLEN] = {0};
 	char query_str_b64[QUERYMAXLEN] = {0};
+	char *cidinfo;
+	char *cidfile;
+	int cidgood = 0;
 
 	s_config *config = config_get_config();
 
@@ -1089,15 +1093,14 @@ static char *construct_querystring(t_client *client, char *originurl, char *quer
 	} else if (config->fas_secure_enabled == 1) {
 
 			if (config->fas_hid) {
-				hash_str(hash, sizeof(hash), client->token);
-				debug(LOG_DEBUG, "hid=%s", hash);
+				debug(LOG_DEBUG, "hid=%s", client->hid);
 
 				get_client_interface(clientif, sizeof(clientif), client->mac);
 				debug(LOG_DEBUG, "clientif: [%s] url_encoded_gw_name: [%s]", clientif, config->url_encoded_gw_name);
 
 				snprintf(query_str, QUERYMAXLEN,
 					"hid=%s%sclientip=%s%sclientmac=%s%sgatewayname=%s%sversion=%s%sgatewayaddress=%s%sgatewaymac=%s%soriginurl=%s%sclientif=%s%sthemespec=%s%s%s%s%s%s",
-					hash, QUERYSEPARATOR,
+					client->hid, QUERYSEPARATOR,
 					client->ip, QUERYSEPARATOR,
 					client->mac, QUERYSEPARATOR,
 					config->url_encoded_gw_name, QUERYSEPARATOR,
@@ -1118,6 +1121,68 @@ static char *construct_querystring(t_client *client, char *originurl, char *quer
 					"?fas=%s",
 					query_str_b64
 				);
+
+				if (config->login_option_enabled >=1) {
+					if (client->cid) {
+						cidgood = 1;
+						safe_asprintf(&cidfile, "%s/ndscids/%s", config->tmpfsmountpoint, client->cid);
+
+						if(access(cidfile, F_OK) != 0) {
+							cidgood=0;
+						}
+						free(cidfile);
+					}
+
+					if (cidgood == 0) {
+						strncpy(cid, query_str_b64+5, 86);
+						client->cid = safe_strdup(cid);
+
+						// Write the new cidfile:
+						safe_asprintf(&cidinfo, "hid=\"%s\"\0", client->hid);
+						write_client_info(msg, sizeof(msg), "write", cid, cidinfo);
+
+						safe_asprintf(&cidinfo, "clientip=\"%s\"\0", client->ip);
+						write_client_info(msg, sizeof(msg), "write", cid, cidinfo);
+
+						safe_asprintf(&cidinfo, "clientmac=\"%s\"\0", client->mac);
+						write_client_info(msg, sizeof(msg), "write", cid, cidinfo);
+
+						safe_asprintf(&cidinfo, "gatewayname=\"%s\"\0", config->http_encoded_gw_name);
+						write_client_info(msg, sizeof(msg), "write", cid, cidinfo);
+
+						safe_asprintf(&cidinfo, "version=\"%s\"\0", VERSION);
+						write_client_info(msg, sizeof(msg), "write", cid, cidinfo);
+
+						safe_asprintf(&cidinfo, "gatewayaddress=\"%s\"\0", config->gw_address);
+						write_client_info(msg, sizeof(msg), "write", cid, cidinfo);
+
+						safe_asprintf(&cidinfo, "gatewaymac=\"%s\"\0", config->gw_mac);
+						write_client_info(msg, sizeof(msg), "write", cid, cidinfo);
+
+						safe_asprintf(&cidinfo, "originurl=\"%s\"\0", originurl);
+						write_client_info(msg, sizeof(msg), "write", cid, cidinfo);
+
+						safe_asprintf(&cidinfo, "clientif=\"%s\"\0", clientif);
+						write_client_info(msg, sizeof(msg), "write", cid, cidinfo);
+
+						safe_asprintf(&cidinfo, "themespec=\"%s\"\0", config->themespec_path);
+						write_client_info(msg, sizeof(msg), "write", cid, cidinfo);
+
+						safe_asprintf(&cidinfo, "%s\0", config->custom_params);
+						write_client_info(msg, sizeof(msg), "parse", cid, cidinfo);
+
+						safe_asprintf(&cidinfo, "%s\0", config->custom_vars);
+						write_client_info(msg, sizeof(msg), "parse", cid, cidinfo);
+
+						safe_asprintf(&cidinfo, "%s\0", config->custom_images);
+						write_client_info(msg, sizeof(msg), "parse", cid, cidinfo);
+
+						safe_asprintf(&cidinfo, "%s\0", config->custom_files);
+						write_client_info(msg, sizeof(msg), "parse", cid, cidinfo);
+
+						free(cidinfo);
+					}
+				}
 			} else {
 				snprintf(querystr, QUERYMAXLEN,
 					"?clientip=%s&gatewayname=%s&redir=%s",
@@ -1128,14 +1193,14 @@ static char *construct_querystring(t_client *client, char *originurl, char *quer
 			}
 
 	} else if (config->fas_secure_enabled == 2 || config->fas_secure_enabled == 3) {
-		hash_str(hash, sizeof(hash), client->token);
-		debug(LOG_DEBUG, "hid=%s", hash);
+
+		debug(LOG_DEBUG, "hid=%s", client->hid);
 
 		get_client_interface(clientif, sizeof(clientif), client->mac);
 		debug(LOG_DEBUG, "clientif: [%s]", clientif);
 		snprintf(querystr, QUERYMAXLEN,
 			"hid=%s%sclientip=%s%sclientmac=%s%sgatewayname=%s%sversion=%s%sgatewayaddress=%s%sgatewaymac=%s%sauthdir=%s%soriginurl=%s%sclientif=%s%sthemespec=%s%s%s%s%s%s",
-			hash, QUERYSEPARATOR,
+			client->hid, QUERYSEPARATOR,
 			client->ip, QUERYSEPARATOR,
 			client->mac, QUERYSEPARATOR,
 			config->url_encoded_gw_name, QUERYSEPARATOR,
@@ -1336,42 +1401,42 @@ static int send_error(struct MHD_Connection *connection, int error)
 
 	switch (error) {
 	case 200:
-		response = MHD_create_response_from_buffer(strlen(page_200), (char *)page_200, MHD_RESPMEM_PERSISTENT);
+		response = MHD_create_response_from_buffer(strlen(page_200), (char *)page_200, MHD_RESPMEM_MUST_COPY);
 		MHD_add_response_header(response, "Content-Type", mimetype);
 		ret = MHD_queue_response(connection, error, response);
 		break;
 
 	case 400:
-		response = MHD_create_response_from_buffer(strlen(page_400), (char *)page_400, MHD_RESPMEM_PERSISTENT);
+		response = MHD_create_response_from_buffer(strlen(page_400), (char *)page_400, MHD_RESPMEM_MUST_COPY);
 		MHD_add_response_header(response, "Content-Type", mimetype);
 		ret = MHD_queue_response(connection, MHD_HTTP_BAD_REQUEST, response);
 		break;
 
 	case 403:
-		response = MHD_create_response_from_buffer(strlen(page_403), (char *)page_403, MHD_RESPMEM_PERSISTENT);
+		response = MHD_create_response_from_buffer(strlen(page_403), (char *)page_403, MHD_RESPMEM_MUST_COPY);
 		MHD_add_response_header(response, "Content-Type", mimetype);
 		ret = MHD_queue_response(connection, MHD_HTTP_FORBIDDEN, response);
 		break;
 
 	case 404:
-		response = MHD_create_response_from_buffer(strlen(page_404), (char *)page_404, MHD_RESPMEM_PERSISTENT);
+		response = MHD_create_response_from_buffer(strlen(page_404), (char *)page_404, MHD_RESPMEM_MUST_COPY);
 		MHD_add_response_header(response, "Content-Type", mimetype);
 		ret = MHD_queue_response(connection, MHD_HTTP_NOT_FOUND, response);
 		break;
 
 	case 500:
-		response = MHD_create_response_from_buffer(strlen(page_500), (char *)page_500, MHD_RESPMEM_PERSISTENT);
+		response = MHD_create_response_from_buffer(strlen(page_500), (char *)page_500, MHD_RESPMEM_MUST_COPY);
 		MHD_add_response_header(response, "Content-Type", mimetype);
 		ret = MHD_queue_response(connection, MHD_HTTP_INTERNAL_SERVER_ERROR, response);
 		break;
 
 	case 501:
-		response = MHD_create_response_from_buffer(strlen(page_501), (char *)page_501, MHD_RESPMEM_PERSISTENT);
+		response = MHD_create_response_from_buffer(strlen(page_501), (char *)page_501, MHD_RESPMEM_MUST_COPY);
 		MHD_add_response_header(response, "Content-Type", mimetype);
 		ret = MHD_queue_response(connection, MHD_HTTP_NOT_IMPLEMENTED, response);
 		break;
 	case 503:
-		response = MHD_create_response_from_buffer(strlen(page_503), (char *)page_503, MHD_RESPMEM_PERSISTENT);
+		response = MHD_create_response_from_buffer(strlen(page_503), (char *)page_503, MHD_RESPMEM_MUST_COPY);
 		MHD_add_response_header(response, "Content-Type", mimetype);
 		ret = MHD_queue_response(connection, MHD_HTTP_INTERNAL_SERVER_ERROR, response);
 		break;
@@ -1406,10 +1471,9 @@ static int send_error(struct MHD_Connection *connection, int error)
 			status_url
 		);
 
-		response = MHD_create_response_from_buffer(strlen(page_511), (char *)page_511, MHD_RESPMEM_MUST_COPY);
+		response = MHD_create_response_from_buffer(strlen(page_511), (char *)page_511, MHD_RESPMEM_MUST_FREE);
 		MHD_add_response_header(response, "Content-Type", mimetype);
 		ret = MHD_queue_response(connection, MHD_HTTP_NETWORK_AUTHENTICATION_REQUIRED, response);
-		free(page_511);
 		free(status_url);
 		break;
 	}
