@@ -135,6 +135,7 @@ termination_handler(int s)
 {
 	static pthread_mutex_t sigterm_mutex = PTHREAD_MUTEX_INITIALIZER;
 	char *fasssl = NULL;
+	int ret;
 
 	debug(LOG_INFO, "Handler for termination caught signal %d", s);
 
@@ -149,7 +150,7 @@ termination_handler(int s)
 	// Check if authmon is already running and if it is, kill it
 	debug(LOG_INFO, "Explicitly killing the authmon daemon");
 	safe_asprintf(&fasssl, "kill $(pgrep -f \"usr/lib/opennds/authmon.sh\") > /dev/null 2>&1");
-	system(fasssl);
+	ret = system(fasssl);
 	free(fasssl);
 
 	auth_client_deauth_all();
@@ -237,9 +238,13 @@ setup_from_config(void)
 {
 	char protocol[8] = {0};
 	char port[8] = {0};
-	char msg[255] = {0};
-	char gwhash[255] = {0};
-	char authmonpid[255] = {0};
+	char msg[256] = {0};
+	char rtest[32] = {0};
+	char *rcmd;
+	const char rtr_fail[] = "-";
+	const char rtr_offline[] = "offline";
+	char gwhash[256] = {0};
+	char authmonpid[16] = {0};
 	char *socket;
 	char *fasurl = NULL;
 	char *fasssl = NULL;
@@ -247,11 +252,12 @@ setup_from_config(void)
 	char *fashid = NULL;
 	char *phpcmd = NULL;
 	char *preauth_dir = NULL;
-	char loginscript[] = "/usr/lib/opennds/libopennds.sh";
-	char gw_name_entityencoded[QUERYMAXLEN] = {0};
-	char gw_name_urlencoded[QUERYMAXLEN] = {0};
+	char libscript[] = "/usr/lib/opennds/libopennds.sh";
+	char gw_name_entityencoded[256] = {0};
+	char gw_name_urlencoded[256] = {0};
 	struct stat sb;
 	time_t sysuptime;
+	time_t now = time(NULL);
 	t_WGFQDN *allowed_wgfqdn;
 	char wgfqdns[1024] = {0};
 	char *dnscmd;
@@ -260,6 +266,12 @@ setup_from_config(void)
 
 	config = config_get_config();
 
+	if (!((stat(libscript, &sb) == 0) && S_ISREG(sb.st_mode) && (sb.st_mode & S_IXUSR))) {
+		debug(LOG_ERR, "Library libopennds does not exist or is not executable");
+		debug(LOG_ERR, "Exiting...");
+		exit(1);
+	}
+
 	debug(LOG_INFO, "tmpfs mountpoint is [%s]", config->tmpfsmountpoint);
 	// Before we do anything else, reset the firewall (cleans it, in case we are restarting after opennds crash)
 	iptables_fw_destroy();
@@ -271,7 +283,7 @@ setup_from_config(void)
 	int outdated = 0;
 	const char *version = MHD_get_version();
 
-	debug(LOG_INFO, "MHD version is %s", version);
+	debug(LOG_NOTICE, "MHD version is %s", version);
 
 	if (sscanf(version, "%d.%d.%d", &major, &minor, &patch) == 3) {
 
@@ -316,13 +328,39 @@ setup_from_config(void)
 
 	config->gw_mac = get_iface_mac(config->gw_interface);
 
-	if (config->gw_mac == NULL) {
+	if (strcmp(config->gw_mac, "00:00:00:00:00:00") == 0 || config->gw_mac == NULL) {
 		debug(LOG_ERR, "Could not get MAC address information of %s, exiting...", config->gw_interface);
 		exit(1);
 	}
 
 	debug(LOG_NOTICE, "Interface %s is at %s (%s)", config->gw_interface, config->gw_ip, config->gw_mac);
 
+	// Check routing configuration
+	safe_asprintf(&rcmd,
+		"/usr/lib/opennds/libopennds.sh gatewayroute \"%s\"",
+		config->gw_interface
+	);
+
+	if (execute_ret_url_encoded(rtest, sizeof(rtest) - 1, rcmd) == 0) {
+		if (strcmp(rtest, rtr_fail) == 0) {
+			debug(LOG_ERR, "Routing configuration is not valid for openNDS, exiting ...");
+			exit(1);
+		} else if (strcmp(rtest, rtr_offline) == 0) {
+			debug(LOG_WARNING, "Upstream gateway is not connected or offline");
+		} else {
+			debug(LOG_NOTICE, "Upstream gateway address/via interface [ %s ]", rtest);
+		}
+	} else {
+		debug(LOG_ERR, "Unable to get routing configuration, exiting ...");
+		exit(1);
+	}
+
+
+	// Warn if Preemptive Authentication is enabled
+	if (config->allow_preemptive_authentication == 1) {
+		debug(LOG_NOTICE, "Preemptive authentication is enabled");
+	}
+
 	// Make sure fas_remoteip is set. Note: This does not enable FAS.
 	if (!config->fas_remoteip) {
 		config->fas_remoteip = safe_strdup(config->gw_ip);
@@ -382,7 +420,7 @@ setup_from_config(void)
 		debug(LOG_DEBUG, "Custom FAS images string [%s]", config->custom_images);
 	}
 
-	// Setup custom FAS images if configured
+	// Setup custom FAS files if configured
 	char fasfile[MAX_BUF] = {0};
 	t_FASFILE *fas_fasfile;
 	if (config->fas_custom_files_list) {
@@ -504,6 +542,7 @@ setup_from_config(void)
 	// Initialize the web server
 	start_mhd();
 	debug(LOG_NOTICE, "Created web server on %s", config->gw_address);
+	debug(LOG_NOTICE, "Maximum Html Page size is [ %llu ] Bytes", HTMLMAXSIZE);
 
 	// Get the ndsctl socket path/filename
 	if (strcmp(config->ndsctl_sock, DEFAULT_NDSCTL_SOCK) == 0) {
@@ -517,10 +556,10 @@ setup_from_config(void)
 	// Check if login script or custom preauth script is enabled
 	if (config->login_option_enabled >= 1) {
 		debug(LOG_NOTICE, "Login option is Enabled using mode %d.\n", config->login_option_enabled);
-		config->preauth = safe_strdup(loginscript);
+		config->preauth = safe_strdup(libscript);
 	} else if (config->login_option_enabled == 0 && config->fas_port == 0 && config->preauth == NULL) {
 		debug(LOG_NOTICE, "Click to Continue option is Enabled.\n");
-		config->preauth = safe_strdup(loginscript);
+		config->preauth = safe_strdup(libscript);
 	} else if (config->login_option_enabled == 0 && config->fas_port == 0 && config->preauth != NULL) {
 		debug(LOG_NOTICE, "Custom PreAuth Script Enabled.\n");
 	} else if (config->login_option_enabled == 0 && config->fas_port >= 1 ) {
@@ -734,6 +773,14 @@ setup_from_config(void)
 		exit(1);
 	}
 
+	// Preload remote files defined in themespec
+	if (config->login_option_enabled == 3) {
+		download_remotes(1);
+		// Initial download is in progress, so sleep for a while before starting watchdog thread.
+		sleep(2);
+		config->remotes_last_refresh = now;
+	}
+
 	debug(LOG_NOTICE, "openNDS is now running.\n");
 }
 
@@ -752,7 +799,7 @@ main_loop(void)
 	// Set up everything we need based on the configuration
 	setup_from_config();
 
-	// Start client statistics and timeout clean-up thread
+	// Start watchdog, client statistics and timeout clean-up thread
 	result = pthread_create(&tid_client_check, NULL, thread_client_timeout_check, NULL);
 	if (result != 0) {
 		debug(LOG_ERR, "FATAL: Failed to create thread_client_timeout_check - exiting");
