@@ -24,6 +24,9 @@
   @author Copyright (C) 2004 Philippe April <papril777@yahoo.com>
   @author Copyright (C) 2006 Benoit Grégoire <bock@step.polymtl.ca>
   @author Copyright (C) 2008 Paul Kube <nodogsplash@kokoro.ucsd.edu>
+  @author Copyright (C) 2015-2021 Modifications and additions by BlueWave Projects and Services <opennds@blue-wave.net>
+  @author Copyright (C) 2021 ndsctl_lock() and ndsctl_unlock() based on code by Linus Lüssing <ll@simonwunderlich.de>
+
  */
 
 #define _GNU_SOURCE
@@ -57,6 +60,7 @@
 #include <string.h>
 #include <pthread.h>
 #include <netdb.h>
+#include <microhttpd.h>
 
 #include "common.h"
 #include "client_list.h"
@@ -65,6 +69,7 @@
 #include "conf.h"
 #include "debug.h"
 #include "fw_iptables.h"
+#include "http_microhttpd_utils.h"
 
 // Defined in main.c
 extern time_t started_time;
@@ -80,6 +85,97 @@ extern unsigned int authenticated_since_start;
 extern int created_httpd_threads;
 extern int current_httpd_threads;
 
+int ndsctl_lock()
+{
+	char *lockfile;
+	s_config *config = config_get_config();
+
+	// Open or create the lock file
+	safe_asprintf(&lockfile, "%s/ndsctl.lock", config->tmpfsmountpoint);
+	config->lockfd = open(lockfile, O_RDWR | O_CREAT, S_IRUSR | S_IWUSR);
+	free(lockfile);
+
+	if (config->lockfd < 0) {
+		debug(LOG_ERR, "CRITICAL ERROR - Unable to open [%s]", lockfile);
+		return 5;
+	}
+
+	if (lockf(config->lockfd, F_TLOCK, 0) == 0) {
+		return 0;
+	} else {
+
+		if (errno != EACCES && errno != EAGAIN) {
+			// persistent error
+			debug(LOG_ERR, "CRITICAL ERROR - Unable to create lock on [%s]", lockfile);
+			close(config->lockfd);
+			return 6;
+		}
+
+		debug(LOG_ERR, "ndsctl is locked by another process");
+		close(config->lockfd);
+		return 4;
+	}
+}
+
+void ndsctl_unlock()
+{
+	int ret;
+	s_config *config = config_get_config();
+	ret = lockf(config->lockfd, F_ULOCK, 0);
+	close(config->lockfd);
+}
+
+
+int download_remotes(int refresh)
+{
+	char *cmd = NULL;
+	int ret;
+	s_config *config = config_get_config();
+
+	if(refresh == 0) {
+		debug(LOG_DEBUG, "Background Checking of remotes for: %s\n", config->themespec_path);
+	} else {
+		debug(LOG_DEBUG, "Background Refreshing of remotes for: %s\n", config->themespec_path);
+	}
+
+	safe_asprintf(&cmd,
+		"/usr/lib/opennds/libopennds.sh download \"%s\" \"%s\" \"%s\" \"%d\" &",
+		config->themespec_path,
+		config->custom_images,
+		config->custom_files,
+		refresh
+	);
+
+	debug(LOG_DEBUG, "Executing system command: %s\n", cmd);
+	ret = system(cmd);
+	free(cmd);
+	return 0;
+}
+
+int write_client_info(char* msg, int msg_len, const char *mode, const char *cid, const char *info)
+{
+	char *cmd = NULL;
+	s_config *config = config_get_config();
+
+	debug(LOG_DEBUG, "Client Info: %s", info);
+	safe_asprintf(&cmd, "/usr/lib/opennds/libopennds.sh '%s' '%s' '%s' '%s'", mode, cid, config->tmpfsmountpoint, info);
+		debug(LOG_DEBUG, "WriteClientInfo command: %s", cmd);
+	if (execute_ret_url_encoded(msg, msg_len - 1, cmd) == 0) {
+		debug(LOG_DEBUG, "Client Info added: %s", info);
+	} else {
+		debug(LOG_INFO, "Failed to write client info [%s] - retrying", info);
+		sleep(1);
+
+		if (execute_ret_url_encoded(msg, msg_len - 1, cmd) == 0) {
+			debug(LOG_DEBUG, "Client Info added: %s", info);
+		} else {
+			debug(LOG_INFO, "Failed to write client info [%s] - giving up", info);
+		}
+	}
+	free (cmd);
+	return 0;
+}
+
 int get_client_interface(char* clientif, int clientif_len, const char *climac)
 {
 	char *clifcmd = NULL;
@@ -263,11 +359,20 @@ get_iface_mac(const char ifname[])
 {
 	char addrbuf[18] = {0};
 	char cmd[128] = {0};
+	s_config *config;
 
-	snprintf(cmd, sizeof(cmd), "ip address | grep -A1 '%s:' | grep 'link/ether ' | awk '{print $2}' | awk -F'/' '{printf $1}'",
-		ifname
+	config = config_get_config();
+
+	if (config->gw_mac == NULL) {
+		config->gw_mac = safe_strdup("00:00:00:00:00:00");
+	}
+
+	snprintf(cmd, sizeof(cmd), "/usr/lib/opennds/libopennds.sh \"gatewaymac\" \"%s\" \"%s\"",
+		ifname,
+		config->gw_mac
 	);
 
+
 	execute_ret(addrbuf, sizeof(addrbuf), cmd);
 	return safe_strdup(addrbuf);
 }
@@ -299,7 +404,10 @@ get_ext_iface(void)
 			}
 		}
 		fclose(input);
-		debug(LOG_ERR, "get_ext_iface(): Failed to detect the external interface after try %d (maybe the interface is not up yet?).  Retry limit: %d", i, NUM_EXT_INTERFACE_DETECT_RETRY);
+		debug(LOG_ERR, "get_ext_iface(): Failed to detect the external interface after try %d (maybe the interface is not up yet?).  Retry limit: %d",
+			i,
+			NUM_EXT_INTERFACE_DETECT_RETRY
+		);
 
 		// Sleep for EXT_INTERFACE_DETECT_RETRY_INTERVAL seconds
 		timeout.tv_sec = time(NULL) + EXT_INTERFACE_DETECT_RETRY_INTERVAL;
@@ -388,11 +496,15 @@ time_t get_system_uptime() {
 	pfp = fopen ("/proc/uptime", "r");
 
 	if (pfp != NULL) {
-		(void) fgets (buf, sizeof(buf), pfp);
-		sysuptime = atol(strtok(buf, "."));
-		debug(LOG_DEBUG, "Operating System Uptime: %li seconds ", sysuptime);
+
+		if(fgets (buf, sizeof(buf), pfp) != NULL) {
+			sysuptime = atol(strtok(buf, "."));
+			debug(LOG_DEBUG, "Operating System Uptime: %li seconds ", sysuptime);
+			fclose (pfp);
+			return sysuptime;
+		}
+
 		fclose (pfp);
-		return sysuptime;
 	}
 
 	debug(LOG_WARNING, "Unable to determine System Uptime.");
@@ -424,6 +536,7 @@ ndsctl_status(FILE *fp)
 	time_t sysuptime;
 	t_WGP *allowed_wgport;
 	t_WGFQDN *allowed_wgfqdn;
+	const char *mhdversion = MHD_get_version();
 
 	config = config_get_config();
 
@@ -444,10 +557,23 @@ ndsctl_status(FILE *fp)
 	format_duration(0, uptimesecs, durationbuf);
 
 	fprintf(fp, "Uptime: %s\n", durationbuf);
-	fprintf(fp, "Gateway Name: %s\n", config->gw_name);
-	fprintf(fp, "Managed interface: %s\n", config->gw_interface);
-	fprintf(fp, "Managed IP range: %s\n", config->gw_iprange);
-	fprintf(fp, "Server listening: http://%s\n", config->gw_address);
+	fprintf(fp, "Gateway Name: [ %s ]\n", config->gw_name);
+	fprintf(fp, "Gateway FQDN: [ %s ]\n", config->gw_fqdn);
+
+	if (strstr(config->gw_iprange, "0.0.0.0/0")) {
+		fprintf(fp, "Managed interface: %s\n", config->gw_interface);
+	} else {
+		fprintf(fp, "Managed interface: %s - IP address range: %s\n", config->gw_interface, config->gw_iprange);
+	}
+
+	fprintf(fp, "MHD Server [ version %s ] listening on: http://%s\n", mhdversion, config->gw_address);
+	fprintf(fp, "Maximum Html Page size is [ %llu ] Bytes\n", HTMLMAXSIZE);
+
+	if (config->allow_preemptive_authentication > 0) {
+		fprintf(fp, "Preemptive Authentication is Enabled\n");
+	} else {
+		fprintf(fp, "Preemptive Authentication is Disabled\n");
+	}
 
 	if (config->binauth) {
 		fprintf(fp, "Binauth Script: %s\n", config->binauth);
